//
// TinyPTC by Gaffer
// http://www.gaffer.org/tinyptc
//
// 2000-12-21: modified by tonic (jetro@iki.fi)
//

// import classes
import java.awt.Color;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.Graphics;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.image.ImageProducer;
import java.awt.image.ImageConsumer;
import java.awt.image.IndexColorModel;


public abstract class tinyptc8
                extends java.applet.Applet
                implements Runnable, ImageProducer
{
    public abstract void main(int width,int height);

    public synchronized void palette(int palette[])
    {
        // create color arrays
        byte r[] = new byte[256];
        byte g[] = new byte[256];
        byte b[] = new byte[256];

        // setup color byte arrays
        for (int i=0; i<256; i++)
        {
            // unpack color integer to r,g,b
            r[i] = (byte) ( (palette[i] & 0x00FF0000) >> 16 );
            g[i] = (byte) ( (palette[i] & 0x0000FF00) >> 8  );
            b[i] = (byte) ( (palette[i] & 0x000000FF)       );
        }

        // create indexed color model
        _model = new IndexColorModel(8,256,r,g,b);
    }


    public synchronized void update(Object pixels)
    {
        // check consumer
        if (_consumer!=null)
        {
            // copy integer pixel data to image consumer
            _consumer.setPixels(0,0,_width,_height,_model,(byte[])pixels,0,_width);

            // notify image consumer that the frame is done
            _consumer.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
        }

        // paint
        paint();
    }

    public void start()
    {
        // check thread
        if (_thread==null)
        {
            // create thread
            _thread = new Thread(this);

            // start thread
            _thread.start();
        }
    }

    public void run()
    {
        // get component size
        Dimension size = getSize();

        // setup data
        _width = size.width;
        _height = size.height;

        // setup color model
        byte[] grayScale = new byte[256];
        for (int a = 0; a < 256; ++a)
            grayScale[a] = (byte)a;
        _model = new IndexColorModel(8, 256, grayScale, grayScale, grayScale);

        // create image using default toolkit
        _image = Toolkit.getDefaultToolkit().createImage(this);

        _alive = true;

        // call user main
        main(_width,_height);
    }

    public void stop()
    {
        // check thread is valid and alive
        if (_thread!=null && _thread.isAlive())
        {
            // stop thread
            _alive = false;
        }

        // null thread
        _thread = null;
    }

    private synchronized void paint()
    {
        // get component graphics object
        Graphics graphics = getGraphics();

        // draw image to graphics context
        graphics.drawImage(_image,0,0,null);
    }

    public synchronized void addConsumer(ImageConsumer ic)
    {
        // register image consumer
        _consumer = ic;

        // set image dimensions
        _consumer.setDimensions(_width,_height);

        // set image consumer hints for speed
        _consumer.setHints(ImageConsumer.TOPDOWNLEFTRIGHT|ImageConsumer.COMPLETESCANLINES|ImageConsumer.SINGLEPASS|ImageConsumer.SINGLEFRAME);

        // set image color model
        _consumer.setColorModel(_model);
    }

    public synchronized boolean isConsumer(ImageConsumer ic)
    {
        // check if consumer is registered
        return true;
    }

    public synchronized void removeConsumer(ImageConsumer ic)
    {
        // remove image consumer
    }

    public void startProduction(ImageConsumer ic)
    {
        // add consumer
        addConsumer(ic);
    }

    public void requestTopDownLeftRightResend(ImageConsumer ic)
    {
        // ignore resend request
    }

    // data
    int _width;
    int _height;
    Image _image;
    Thread _thread;
    ImageConsumer _consumer;
    IndexColorModel _model;

    protected boolean _alive;
}
